/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package BCE;

import gurobi.*;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

public class CallbackBCD<K> extends GRBCallback {
  private GRBVar[]   vars;
  private FileWriter logfile;
  private Graph G;
  private HashMap<Edge<K>, GRBVar> edgeVariables;

  public CallbackBCD(GRBVar[] vars, FileWriter logfile, Graph g, HashMap<Edge<K>, GRBVar> edgeVariables) {
    this.vars = vars;
    this.logfile = logfile;
    this.G = g;   
    this.edgeVariables = edgeVariables; 
  }
  
protected void callback(){   
     try {
         if (where == GRB.Callback.MIPSOL){             
             
            Set<Vertex<K>> verticesInG = G.getVertices();
            
            Graph g2 = new Graph(verticesInG);
            for(Edge e : edgeVariables.keySet()){
                if(this.getSolution(edgeVariables.get(e))>= 0.9999){
                    Vertex<K>[] nodeArray = e.getNodeArray();
                    g2.addEdge(nodeArray[0],nodeArray[1]);
                }
            }

            Set<Set<Vertex<K>>> allColorConflicts = g2.allColorConflicts();
            for(Set<Vertex<K>> colorConflicts : allColorConflicts){
                GRBLinExpr newLazy = new GRBLinExpr();
                Set<Vertex<K>> alreadyDone = new HashSet();
                for(Vertex<K> v : colorConflicts){
                    alreadyDone.add(v);
                    for(Vertex<K> w : colorConflicts){
                        if(!alreadyDone.contains(w)){
                            Edge<K> e = new Edge(v,w);
                            newLazy.addTerm(1.0,edgeVariables.get(e));
                        }
                    }
                }
//                System.out.println("!!!!Lazy Color Constraint added !!!!"+"\n"+
//                        newLazy.getCoeff(0)+newLazy.getVar(0).get(GRB.StringAttr.VarName)+" "+
//                        newLazy.getCoeff(1)+newLazy.getVar(1).get(GRB.StringAttr.VarName)+" "+
//                        newLazy.getCoeff(2)+newLazy.getVar(2).get(GRB.StringAttr.VarName)+" "+
//                        newLazy.getCoeff(3)+newLazy.getVar(3).get(GRB.StringAttr.VarName)+" "+
//                        newLazy.getCoeff(4)+newLazy.getVar(4).get(GRB.StringAttr.VarName)+" "+
//                        newLazy.getCoeff(5)+newLazy.getVar(5).get(GRB.StringAttr.VarName));
//                this.addLazy(newLazy,GRB.LESS_EQUAL, 3.0);
            }                
         }
     } catch (GRBException e) {
      System.out.println("Error code: " + e.getErrorCode());
      System.out.println(e.getMessage());
      e.printStackTrace();
    } catch (Exception e) {
      System.out.println("Error during callback");
      e.printStackTrace();
    }
}

}