/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package BCE;

import gurobi.*;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

public class CallbackBCE<K> extends GRBCallback {
  private GRBVar[]   vars;
  private FileWriter logfile;
  private Graph G;
  private HashMap<Edge<K>, GRBVar> edgeVariables;
  private HashMap<Edge<K>, GRBVar> missingEdgeVariables;

  public CallbackBCE(GRBVar[] vars, FileWriter logfile, Graph g, HashMap<Edge<K>, GRBVar> edgeVariables, HashMap<Edge<K>, GRBVar> missingEdgeVariables) {
    this.vars = vars;
    this.logfile = logfile;
    this.G = g;   
    this.edgeVariables = edgeVariables;
    this.missingEdgeVariables = missingEdgeVariables;    
  }
  
protected void callback(){   
     try {
         if (where == GRB.Callback.MIPSOL){     
             
            Set<Vertex<K>> verticesInG = G.getVertices();
            
            Graph g2 = new Graph(verticesInG);
            for(Edge e : edgeVariables.keySet()){
                if(this.getSolution(edgeVariables.get(e))>= 0.9999){
                    Vertex<K>[] nodeArray = e.getNodeArray();
                    g2.addEdge(nodeArray[0],nodeArray[1]);
                }
            }
            for(Edge e : missingEdgeVariables.keySet()){
                if(this.getSolution(missingEdgeVariables.get(e))>= 0.9999){
                    Vertex<K>[] nodeArray = e.getNodeArray();
                    g2.addEdge(nodeArray[0],nodeArray[1]);
                }                 
            }

            Set<Set<Edge<K>>> P3s = g2.getP3s();

            for(Set<Edge<K>> newP3 : P3s){
                Set<Vertex<K>> verticesInP3s = new HashSet<>();
                for(Edge<K> p : newP3){
                    verticesInP3s.addAll(p.getNodes());
                }
                Iterator vertexIterator = verticesInP3s.iterator();
                Vertex<K> u = (Vertex<K>) vertexIterator.next();
                Vertex<K> v = (Vertex<K>) vertexIterator.next();
                Vertex<K> w = (Vertex<K>) vertexIterator.next();
                Edge<K> uv = new Edge(u,v);
                Edge<K> vw = new Edge(v,w);
                Edge<K> uw = new Edge(u,w);
                Set<Edge<K>> edges = new HashSet<>();
                edges.add(uv);
                edges.add(vw);
                edges.add(uw);
                for(Edge<K> e : edges){
                    GRBLinExpr newLazy = new GRBLinExpr();
                    for(Edge<K> e2 : edges){
                        if(edgeVariables.keySet().contains(e2)){
                            if(e2.equals(e)){
                                newLazy.addTerm(-1.0,edgeVariables.get(e2));
                            }
                            else newLazy.addTerm(1.0,edgeVariables.get(e2));
                        }
                        else if(e2.equals(e)){
                            newLazy.addTerm(-1.0,missingEdgeVariables.get(e2));
                            }
                            else newLazy.addTerm(1.0,missingEdgeVariables.get(e2));                       
                    }  
//                    System.out.println("!!   Lazy P3 Constraint added !!"+"\n"+
//                            newLazy.getCoeff(0)+newLazy.getVar(0).get(GRB.StringAttr.VarName)+"  "+
//                            newLazy.getCoeff(1)+newLazy.getVar(1).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(2)+newLazy.getVar(2).get(GRB.StringAttr.VarName));
                    this.addLazy(newLazy, GRB.LESS_EQUAL, 1.0);                    
                }
            }
            if(P3s.isEmpty()){
                Set<Set<Vertex<K>>> allColorConflicts = g2.allColorConflicts();
                for(Set<Vertex<K>> colorConflicts : allColorConflicts){
                    GRBLinExpr newLazy = new GRBLinExpr();
                    Set<Vertex<K>> alreadyDone = new HashSet();
                    for(Vertex<K> v : colorConflicts){
                        alreadyDone.add(v);
                        for(Vertex<K> w : colorConflicts){
                            if(!alreadyDone.contains(w)){
                                Edge<K> e = new Edge(v,w);
                                if(edgeVariables.keySet().contains(e)){
                                    newLazy.addTerm(1.0,edgeVariables.get(e));
                                }
                                else newLazy.addTerm(1.0,missingEdgeVariables.get(e));
                            }
                        }
                    }
//                    System.out.println("!!!!Lazy Color Constraint added !!!!"+"\n"+
//                            newLazy.getCoeff(0)+newLazy.getVar(0).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(1)+newLazy.getVar(1).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(2)+newLazy.getVar(2).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(3)+newLazy.getVar(3).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(4)+newLazy.getVar(4).get(GRB.StringAttr.VarName)+" "+
//                            newLazy.getCoeff(5)+newLazy.getVar(5).get(GRB.StringAttr.VarName));
                    this.addLazy(newLazy,GRB.LESS_EQUAL, 3.0);
                }                
            }
         }
     } catch (GRBException e) {
      System.out.println("Error code: " + e.getErrorCode());
      System.out.println(e.getMessage());
      e.printStackTrace();
    } catch (Exception e) {
      System.out.println("Error during callback");
      e.printStackTrace();
    }
}

}