package BCE;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.charset.Charset;
import java.util.*;

public class FileWriter<K> {
    
    Charset utf8 = java.nio.charset.StandardCharsets.UTF_8;
    
    private static FileWriter instance = null;
    
    private FileWriter(){}
    
    public static FileWriter getInstance(){
        if(instance == null){
            instance = new FileWriter();
        }
        return instance;
    }
    
    public boolean writeToFile(String path, List<String> lines){
        try{
            Files.write(Paths.get(path), lines, utf8);
            return true;
        } catch (Exception e){
            System.out.println(e.getMessage());
            return false;
        }
    }
    
    public String edgeToLine(Edge<K> e){
        Set<Vertex<K>> vertices = e.getNodes();
        String line = vertices.toArray()[0] + " " + vertices.toArray()[1];
        return line;
    }
    
    public String whiteVertexToLine(Vertex<K> v){
        String line = "w "+v;
        return line;
    }
    
    public boolean writeBicoloredGraph(String path, Graph G){
        try{     
            List<String> output = new ArrayList();
            Set<Vertex<K>> whiteVertices = G.getWhiteVertices();
            Set<Edge<K>> edges = G.getEdges();
            for(Vertex<K> w : whiteVertices){
                output.add(whiteVertexToLine(w));
            }
            for(Edge<K> e : edges){
                output.add(edgeToLine(e));
            }
            writeToFile(path, output);           
            return true;
        } catch (Exception e){
            System.out.println(e.getMessage());
            return false;
        }
    }
    
    
    
       
    
     
}

