package BCE;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

public class GraphImport {
    
    private static GraphImport instance = null;
    
    private GraphImport(){}
    
    public static GraphImport getInstance(){
        if(instance == null){
            instance = new GraphImport();
        }
        return instance;
    }
    
    
    
    public Graph<String> readGraphString(String file){
        Set<Edge<String>> edges = new HashSet();
        Set<Vertex<String>> vertices = new HashSet();
        try {
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            for(String line; (line = bufferedReader.readLine()) != null; ){
                String modifiedLine = line.replace("\t", " ");
                String[] edgeNodes = modifiedLine.split(" ");
                if(!(edgeNodes.length==0) && !edgeNodes[0].contains("%")){  // % signals comments in data sets
                    if(!edgeNodes[0].equals(edgeNodes[1])){
                        Edge<String> edge = new Edge(edgeNodes[0],edgeNodes[1]);
                        edges.add(edge);
                    }                     
                    vertices.add(new Vertex(edgeNodes[0]));
                    vertices.add(new Vertex(edgeNodes[1]));
                }
            }
        } catch (IOException e) {
            System.out.println(e.getMessage());
        }
        Graph<String> G = new Graph(vertices);
        for(Edge<String> e : edges){
            Vertex<String>[] vertices_e = e.getNodeArray();
            G.addEdge(vertices_e[0], vertices_e[1]);
        }
        return G;
    }
    
    
    public Graph<String> readBipartiteBicoloredGraphString(String file){
        Set<Edge<String>> edges = new HashSet();
        Set<Vertex<String>> vertices = new HashSet();
        try {
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            for(String line; (line = bufferedReader.readLine()) != null; ){
                String modifiedLine = line.replace("\t", " ");
                String[] edgeNodes = modifiedLine.split(" ");
                if(!(edgeNodes.length==0) && !edgeNodes[0].contains("%")){  // % signals comments in data sets
                    if(!edgeNodes[0].equals(edgeNodes[1])){
                        Vertex<String> v1 = new Vertex<>(edgeNodes[0],"b");
                        Vertex<String> v2 = new Vertex<>(edgeNodes[1],"w");
                        Edge<String> edge = new Edge(v1,v2);
                        edges.add(edge);
                        vertices.add(v1);
                        vertices.add(v2);
                    }                     
                }
            }
        } catch (IOException e) {
            System.out.println(e.getMessage());
        }
        Graph<String> G = new Graph(vertices);
        for(Edge<String> e : edges){
            Vertex<String>[] vertices_e = e.getNodeArray();
            G.addEdge(vertices_e[0], vertices_e[1]);
        }
        return G;
    }
   
   
    public Graph<String> readBicoloredGraphString(String file){
        Set<Edge<String>> edges = new HashSet();
        Set<Vertex<String>> vertices = new HashSet();
        HashMap<String,Vertex<String>> vertexMap = new HashMap();
        try {
            FileReader fileReader = new FileReader(file);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            for(String line; (line = bufferedReader.readLine()) != null; ){
                String modifiedLine = line.replace("\t", " ");
                String[] edgeNodes = modifiedLine.split(" ");
                if(!(edgeNodes.length==0) && !edgeNodes[0].contains("%")){  // % signals comments in data sets
                    if(edgeNodes[0].equals("w")){
                        Vertex<String> w = new Vertex<>(edgeNodes[1],"w");
                        vertexMap.put(edgeNodes[1],w);
                        vertices.add(w);
                    }
                    if(edgeNodes[0].equals("e")){
                        if(!vertexMap.keySet().contains(edgeNodes[1])){
                            Vertex<String> b = new Vertex<>(edgeNodes[1],"b");
                            vertexMap.put(edgeNodes[1],b);
                            vertices.add(b);
                        }
                        if(!vertexMap.keySet().contains(edgeNodes[2])){
                            Vertex<String> b = new Vertex<>(edgeNodes[2],"b");
                            vertexMap.put(edgeNodes[2],b);
                            vertices.add(b);
                        }
                        if(!edgeNodes[1].equals(edgeNodes[2])){
                            Vertex<String> v1 = vertexMap.get(edgeNodes[1]);
                            Vertex<String> v2 = vertexMap.get(edgeNodes[2]);
                            Edge<String> edge = new Edge(v1,v2);
                            edges.add(edge);
                        } 
                    }                  
                }
            }
        } catch (IOException e) {
            System.out.println(e.getMessage());
        }
        Graph<String> G = new Graph(vertices);
        for(Edge<String> e : edges){
            Vertex<String>[] vertices_e = e.getNodeArray();
            G.addEdge(vertices_e[0], vertices_e[1]);
        }
        return G;
    }
   
   
   
        
}
