
package BCE;


import java.util.*;
import java.util.concurrent.TimeUnit;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.nio.charset.Charset;


public class RunILPs {
    
    
    /*
    * Solves BCD for the input graph, returns a tuple of the solution set and statistics string and writes number of trivial components to file
    */
    public static Tuple<Set<Edge<String>> , ArrayList<String>> runDeletion(Graph input, String inputGraphString){
        ILPs ILP = new ILPs();
        FileWriter fileWriter = FileWriter.getInstance();
        Set<Graph> components = input.getConnectedComponents();
        
         Set<Edge<String>> solutionSet = new HashSet();

             ArrayList<String> resultsDeletion = new ArrayList();
             String firstLineDeletion = "%" + 
                     "n_b"+"\t"+
                     "n_w"+"\t"+
                     "m"+"\t"+
                     "solved?"+"\t"+
                     "time"+"\t"+
                     "k"+"\t"+
                     "#clu"+"\t"+
                     "#sing"+"\t"+
                     "#isoE"+"\t"+
                     "#moBl"+"\t"+
                     "#moWh"+"\t"+
                     "#domB"+"\t"+
                     "#domW"+"\t"+
                     "cluHisto"
                     ;
             resultsDeletion.add(firstLineDeletion);

             Set<Graph> hugeComponents = new HashSet();
             Set<Graph> trivialComponents = new HashSet();
             
             long StartTime = System.nanoTime();
                 for(Graph g : components){
                     Integer vertexCount = g.getVertexCount();
                     if(vertexCount < 800){                         
                         if(g.getBlackVertices().size() >= 2 && g.getWhiteVertices().size() >= 2){         
                            System.out.println("Component of size "+vertexCount);

                            Tuple<Set<Edge<String>> , String > subsol = ILP.solveBCD(g);
                            solutionSet.addAll(subsol.getFirst());
                            System.out.println(subsol.getSecond());

                            resultsDeletion.add(subsol.getSecond());
                         }
                         else {
                             trivialComponents.add(g);
                         }
                     }
                     else {
                         hugeComponents.add(g);
                     }
                 }
             long EndTime = System.nanoTime();
             long Runtime = EndTime - StartTime;
             long convertedRuntime = TimeUnit.SECONDS.convert(Runtime, TimeUnit.NANOSECONDS);

             System.out.println("Total Runtime: "+ convertedRuntime + " seconds");

             System.out.println(solutionSet);        
             
             ArrayList<String> noILPComponents = new ArrayList();
             noILPComponents.add("hugeComponents: " + hugeComponents.size());
             noILPComponents.add("trivialComponents: " + trivialComponents.size());
             
             String trivialCompsResult = "res/deletion_results/trivialComps_" + inputGraphString + "_deletion";
             fileWriter.writeToFile(trivialCompsResult, noILPComponents);
        
             return new Tuple(solutionSet,resultsDeletion);
    }
    
    
    
    /*
    * Solves BCE for the input graph, returns a tuple of the solution set and statistics string and writes number of trivial components to file
    */
    public static Tuple<Set<Edge<String>> , ArrayList<String>> runEditing(Graph input, String inputGraphString){
        ILPs ILP = new ILPs();
        FileWriter fileWriter = FileWriter.getInstance();
        Set<Graph> components = input.getConnectedComponents();
        
         Set<Edge<String>> solutionSet = new HashSet();

             ArrayList<String> resultsEditing = new ArrayList();
             String firstLineEditing = "%" + 
                     "n_b"+"\t"+
                     "n_w"+"\t"+
                     "m"+"\t"+
                     "solved?"+"\t"+
                     "time"+"\t"+
                     "kdel"+"\t"+
                     "kins"+"\t"+
                     "#clu"+"\t"+
                     "#sing"+"\t"+
                     "#isoE"+"\t"+
                     "#moBl"+"\t"+
                     "#moWh"+"\t"+
                     "#domB"+"\t"+
                     "#domW"+"\t"+
                     "cluHisto"+"\t"+
                     "#noClu"+"\t"+
                     "#invClu"
                     ;
             resultsEditing.add(firstLineEditing);
             
             Set<Graph> hugeComponents = new HashSet();
             Set<Graph> trivialComponents = new HashSet();

             long StartTime = System.nanoTime();
                 for(Graph g : components){
                    Integer vertexCount = g.getVertexCount();
                     if(vertexCount < 800){                         
                         if(g.getBlackVertices().size() >= 2 && g.getWhiteVertices().size() >= 2){         
                            System.out.println("Component of size "+vertexCount);

                            Tuple<Set<Edge<String>> , String > subsol = ILP.solveBCE(g);
                            solutionSet.addAll(subsol.getFirst());
                            System.out.println(subsol.getSecond());

                            resultsEditing.add(subsol.getSecond());
                         }
                         else {
                             trivialComponents.add(g);
                         }
                     }
                     else {
                         hugeComponents.add(g);
                     }
                 }
             long EndTime = System.nanoTime();
             long Runtime = EndTime - StartTime;
             long convertedRuntime = TimeUnit.SECONDS.convert(Runtime, TimeUnit.NANOSECONDS);

             System.out.println("Total Runtime: "+ convertedRuntime + " seconds");

             System.out.println(solutionSet);        
             
             ArrayList<String> noILPComponents = new ArrayList();
             noILPComponents.add("hugeComponents: " + hugeComponents.size());
             noILPComponents.add("trivialComponents: " + trivialComponents.size());
             
             String trivialCompsResult = "res/editing_results/trivialComps_" + inputGraphString + "_editing";
             fileWriter.writeToFile(trivialCompsResult, noILPComponents);
        
             return new Tuple(solutionSet,resultsEditing);
    }
        
    
    
    // Computes input graph stats and writes them to a file
    public static void computeComponents(Graph input, String inputGraphString){
        
        FileWriter fileWriter = FileWriter.getInstance();
        Set<Graph> components = input.getConnectedComponents();
        
        

             ArrayList<String> result = new ArrayList();
             String firstLine = "%" + 
                     "n"+"\t"+
                     "n_b"+"\t"+
                     "n_w"+"\t"+
                     "m"
                     ;
             result.add(firstLine);
             
                 for(Graph g : components){
                    Integer vertexCount = g.getVertexCount();
                     if(vertexCount < 100){                         
                         if(g.getBlackVertices().size() >= 2 && g.getWhiteVertices().size() >= 2){         
                            System.out.println("Component of size "+vertexCount);
                            Set<Vertex> blackVerts = g.getBlackVertices();
                            Set<Vertex> whiteVerts = g.getWhiteVertices();
                            Set<Edge> edges = g.getEdges();

                            Integer blackCount = blackVerts.size();
                            Integer whiteCount = whiteVerts.size();
                            Integer edgeCount = edges.size();


                            result.add(vertexCount+"\t"+
                                      blackCount+"\t"+
                                      whiteCount+"\t"+
                                      edgeCount);
                        }
                     }
                 }

             
             String CompsResult = "res/editing_results/InputComponents_" + inputGraphString + "_editing";
             fileWriter.writeToFile(CompsResult, result);
        
    }
    


    public static void main(String[] args) {       
        
         GraphImport importer = GraphImport.getInstance();
         FileWriter fileWriter = FileWriter.getInstance();

     
        List<String> data = new ArrayList();
        data.add("cacao-poplar-0-ocd.graph");
//        data.add("ricinus-poplar-0-ocd.graph");
//        data.add("cacao-poplar-1e-140-ocd.graph");
//        data.add("ricinus-poplar-1e-140-ocd.graph");
//        data.add("cacao-poplar-1e-80-ocd.graph");
//        data.add("ricinus-poplar-1e-80-ocd.graph");

        
        for(String i : data){
            String inputGraphString = i;
            Graph<String> InputGraph = importer.readBicoloredGraphString("res/"+inputGraphString);

            Tuple<Set<Edge<String>> , ArrayList<String>> deletionResults = runDeletion(InputGraph, inputGraphString);
            String ResultFileDeletion = "res/deletion_results/" + inputGraphString + "_deletion";
            fileWriter.writeToFile(ResultFileDeletion, deletionResults.getSecond());
            
            Tuple<Set<Edge<String>> , ArrayList<String>> editingResults = runEditing(InputGraph,inputGraphString);
            String ResultFileEditing = "res/editing_results/" + inputGraphString + "_editing";
            fileWriter.writeToFile(ResultFileEditing, editingResults.getSecond());
        }                
        
    }
       
     
}

