package tests;

import BCE.Graph;
import BCE.GraphImport;
import BCE.Vertex;
import java.*;
import java.util.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.Test;

public class GraphTest {
    
    Graph<java.lang.Integer> G;
    Graph<String> H;
    Set<Vertex<Integer>> vertices = new HashSet<>();
    GraphImport importer = GraphImport.getInstance();
        
    Vertex<Integer> v1 = new Vertex(1,"b");
    Vertex<Integer> v2 = new Vertex(2,"b");
    Vertex<Integer> v3 = new Vertex(3,"b");
    Vertex<Integer> v4 = new Vertex(4,"w");
    
    /*
    * Uses Paw-Graph for Testing.
    */
    
    @Before
    public void setUp(){
        vertices.add(v1);
        vertices.add(v2);
        vertices.add(v3);
        vertices.add(v4); 
        G = new Graph(vertices);
        G.addEdge(v1,v2);
        G.addEdge(v2,v3);
        G.addEdge(v2,v4);
        G.addEdge(v3,v4);
    }
    
    @Test
    public void InitiallizeTest(){
        assertEquals(G.getVertices().size(),4);
    }
    
    @Test
    public void addEdgeTest(){
        assertEquals(G.getNeighbors(v1).size(),1);
        assertEquals(G.getNeighbors(v2).size(),3);
        assertEquals(G.getNeighbors(v3).size(),2);
        assertEquals(G.getNeighbors(v4).size(),2);
        assertEquals(G.getEdges().size(),4);
    }
    
    

    
    
    
    @Test 
    public void ImporterTest(){
        H = importer.readGraphString("res/paw_graph");
        assertEquals(H.getVertices().size(),4);
        assertEquals(H.getNeighbors("1").size(),1);
        assertEquals(H.getNeighbors("2").size(),3);
        assertEquals(H.getNeighbors("3").size(),2);
        assertEquals(H.getNeighbors("4").size(),2);
        assertEquals(H.getEdges().size(),4);
    }
    
    
    /*
    @Test
    public void triangleTest(){
        H = importer.readGraphString("res/triangle_graph.txt");
        Set<Set<Edge<String>>> triangles = H.getTriangles();
        System.out.println(triangles);
    }
    */
    
    /*
    @Test 
    public void ImporterTest2(){
        Graph<String> J = importer.readBipartiteBicoloredGraphString("res/cacao-poplar-0.graph");
        Set<Graph> components = J.getConnectedComponents();
        System.out.println(Arrays.toString(components.toArray()));
    }
    */
    
    
    /*
    @Test 
    public void ImporterTest3(){
        Graph<String> J = importer.readBicoloredGraphString("res/cacao-poplar-0-ocd.graph");
        Set<Graph> components = J.getConnectedComponents();
        System.out.println(Arrays.toString(components.toArray()));
    }
    
    */
    
    /*
    @Test 
    public void CopyTest(){
        System.out.println("CopyTest\n");
        Graph<String> K = importer.readGraphString("res/paw_graph");
        Graph<String> Kcopy = K.copyGraph();
        System.out.println(K);
        System.out.println(Kcopy);
        Kcopy.deleteEdgeByValue("1","2");
        System.out.println(K);
        System.out.println(Kcopy);   
        Kcopy.deleteVertexByValue("1");
        System.out.println(K);
        System.out.println(Kcopy);
    }
    */
    
    
    /*
    @Test 
    public void DistortionTest(){
        System.out.println("DistortionTest\n");
        Graph<Integer> J = G.createTestGraph(3, 5, 34);
        System.out.println(J);
        Graph<Integer> N = J.distortGraph(4, 34);
        System.out.println(N);
    }
    */
    
    
    /*
    @Test 
    public void ClusterTypeTest(){
        System.out.println("ClusterTypeTest\n");
        Graph<Integer> J = G.createTestGraph(3, 5, 20);
        System.out.println(J);
        Set<Graph> components = J.getConnectedComponents();
        for(Graph c : components){
            if(c.isCluster()){
                System.out.println(c+" is a cluster");
                if(c.isSingleton()){
                    System.out.println(c+ " is a singleton");
                }
                if(c.isIsolatedEdge()){
                    System.out.println(c+ " is an isolated edge");
                }
                if(c.isMonochromaticBlackCluster()){
                    System.out.println(c+ " is a monochromatic black cluster");
                }
                if(c.isMonochromaticWhiteCluster()){
                    System.out.println(c+ " is a monochromatic white cluster");
                }
                if(c.isValidBlackDominatedCluster()){
                    System.out.println(c+ " is a valid black-dominated cluster");
                }
                if(c.isValidWhiteDominatedCluster()){
                    System.out.println(c+ " is a valid white-dominated cluster");
                }
            }
            else System.out.println(c+" not a cluster!!");
        }
    }
    */
    
    
    @Test 
    public void ClusterTypeTest2(){
        System.out.println("ClusterTypeTest2\n");
        Graph<Integer> I = G.createTestGraph(10, 5, 20);
        Graph<Integer> J = I.distortGraph(4, 20);
        System.out.println(J);
        Set<Graph> components = J.getConnectedComponents();
        for(Graph c : components){
            if(c.isCluster()){
           //     System.out.println(c+" is a cluster");
                if(c.isSingleton()){
                    System.out.println(c+ " is a singleton");
                }
                if(c.isIsolatedEdge()){
                    System.out.println(c+ " is an isolated edge");
                }
                if(c.isMonochromaticBlackCluster()){
                    System.out.println(c+ " is a monochromatic black cluster");
                }
                if(c.isMonochromaticWhiteCluster()){
                    System.out.println(c+ " is a monochromatic white cluster");
                }
                if(c.isValidBlackDominatedCluster()){
                    System.out.println(c+ " is a valid black-dominated cluster");
                }
                if(c.isValidWhiteDominatedCluster()){
                    System.out.println(c+ " is a valid white-dominated cluster");
                }
            }
            else System.out.println(c+" not a cluster!!");
        }
    }
    
    /*    
    @Test 
    public void isCliqueTest(){
        Set<Vertex<Integer>> subgraph = new HashSet<>();
        Vertex<Integer> v1 = new Vertex(2);
        Vertex<Integer> v2 = new Vertex(3);
        Vertex<Integer> v3 = new Vertex(4);
        subgraph.add(v1);
        subgraph.add(v2);
        subgraph.add(v3);
        boolean isClique = G.isClique(subgraph);
        assertTrue(isClique);
    }
    */
    
     /*    
    @Test 
    public void isClusterTest(){
        Graph<Integer> graph = new Graph<>();
        Vertex<Integer> v1 = new Vertex(2,"b");
        Vertex<Integer> v2 = new Vertex(3,"b");
        Vertex<Integer> v3 = new Vertex(4,"w");
        graph.addVertex(v1);
        graph.addVertex(v2);
        graph.addVertex(v3);
        graph.addEdge(v1,v2);
        graph.addEdge(v2,v3);
        graph.addEdge(v1,v3);
        boolean isCluster = graph.isCluster();
        assertTrue(isCluster);
    }
    */
    
    
    
    /*
    @Test
    public void BFSTest(){
        Graph<String> K = importer.readGraphString("res/test_graph.txt");
        Vertex<String> start = K.getVertex("7");
        Set<Vertex<String>> V = K.BFS(start);
      //  System.out.println(V.toString());
    }
    
    @Test
    public void CCTest(){
        Graph<String> I = importer.readGraphString("res/test_graph.txt");
        Set<Graph> components = I.getConnectedComponents();
        assertEquals(components.size(),3);
        System.out.println(components.toString());
    }
    */
    
    /*
    @Test 
    public void getP3sTest(){
        Set<Set<Edge<Integer>>> p3s = G.getP3s();
        assertEquals(p3s.size(),2);
    }
    */
    
    
    /*
    @Test 
    public void testGraphTest(){
        Graph<Integer> L = G.createTestGraph(3, 5, 20);
        System.out.println(L);
        Set<Graph> components = L.getConnectedComponents();
        System.out.println(L);
        assertEquals(components.size(),3);
        for(Graph C : components){
            assertTrue(C.getVertices().size()<=5);
        }  
    }    
    */
    
    /*
    @Test 
    public void colorConflictTest(){
        System.out.println("Color Conflict Test");
        Vertex<Integer> v1 = new Vertex(1,"b");
        Vertex<Integer> v2 = new Vertex(2,"w");
        Vertex<Integer> v3 = new Vertex(3,"w");
        Vertex<Integer> v4 = new Vertex(4,"w");
        Vertex<Integer> v5 = new Vertex(5,"w");
        Vertex<Integer> v6 = new Vertex(6,"b");
        Vertex<Integer> v7 = new Vertex(7,"b");
        Graph<Integer> K = new Graph<>();
        K.addVertex(v1);
        K.addVertex(v2);
        K.addVertex(v3);
        K.addVertex(v4);
        K.addVertex(v5);
        K.addVertex(v6);
        K.addVertex(v7);
        K.addEdge(v1,v2);
        K.addEdge(v1,v3);
        K.addEdge(v1,v4);
        K.addEdge(v1,v5);
        K.addEdge(v1,v6);
        K.addEdge(v1,v7);     
        K.addEdge(v3,v6);
        Set<Vertex<Integer>> conflicts = K.noColorConflicts();
        //System.out.println(conflicts);
        assertEquals(conflicts.size(),4);
    }  
    */
    
    /*
    @Test 
    public void allColorConflictTest(){
        System.out.println("All Color Conflict Test");
        Graph<String> L = importer.readBicoloredGraphString("res/colorTest_graph.txt");
        System.out.println(L);
        Set<Set<Vertex<String>>> conflicts = L.allColorConflicts();
        for(Set<Vertex<String>> conflict : conflicts){
            System.out.println(conflict);
        }
     //   assertEquals(conflicts.size(),4);
    }
    */
    

    
}
